@@ -24,10 +24,7 @@ sub _dont_emit_initializing_messages {
 
     my $message = $msgobj->text;
     if ($message !~ m/^Re-creating/) {
-        $dsobj->message_callback($msgtype, undef);
-        my $msg_method = $msgtype . '_message';
-        $dsobj->$msg_method($message);
-        $dsobj->message_callback($msgtype, \&_dont_emit_initializing_messages);
+        die;
     }
 }
 
