@@ -10,7 +10,7 @@ use Getopt::Long;
 use Term::ANSIColor;
 require Text::Wrap;
 
-our $VERSION = "0.35"; # UR $VERSION;
+our $VERSION = "0.36"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
@@ -20,7 +20,7 @@ UR::Object::Type->define(
         is_input            => { is => 'Boolean', is_optional => 1 },
         is_output           => { is => 'Boolean', is_optional => 1 },
         is_param            => { is => 'Boolean', is_optional => 1 },
-        shell_args_position => { is => 'Integer', is_optional => 1, 
+        shell_args_position => { is => 'Integer', is_optional => 1,
                                  doc => 'when set, this property is a positional argument when run from a shell' },
     ],
     has_optional => [
@@ -38,9 +38,25 @@ eval {
     binmode STDERR, ":utf8";
 };
 
+# Override method in UR::Object to support error_die and error_rv_false
+sub validate_subscription {
+    my $self = shift;
+    my $subscription_property = shift;
+
+    my $retval = $self->SUPER::validate_subscription($subscription_property, @_);
+    return $retval if $retval;
+
+    unless ( defined($subscription_property) and $subscription_property eq 'error_die') {
+        $subscription_property = '(undef)' unless defined ($subscription_property);
+        Carp::croak("Unrecognized subscription aspect '$subscription_property'");
+    }
+
+    return 1;
+}
+
 sub _init_subclass {
     # Each Command subclass has an automatic wrapper around execute().
-    # This ensures it can be called as a class or instance method, 
+    # This ensures it can be called as a class or instance method,
     # and that proper handling occurs around it.
     my $subclass_name = $_[0];
     no strict;
@@ -78,11 +94,11 @@ sub shortcut {
 
 sub execute {
     # This is a wrapper for real execute() calls.
-    # All execute() methods are turned into _execute_body at class init, 
-    # so this will get direct control when execute() is called. 
+    # All execute() methods are turned into _execute_body at class init,
+    # so this will get direct control when execute() is called.
     my $self = shift;
 
-    #TODO handle calls to SUPER::execute() from another execute().    
+    #TODO handle calls to SUPER::execute() from another execute().
 
     # handle calls as a class method
     my $was_called_as_class_method = 0;
@@ -113,7 +129,21 @@ sub execute {
         return;
     }
 
-    my $result = $self->_execute_body(@_);
+    my $result;
+    eval { $result = $self->_execute_body(@_); };
+    my $error = $@;
+    if ($error or not $result) {
+        my %error_data;
+
+        $error_data{die_message} = defined($error) ? $error:'';
+        $error_data{error_message} = defined($self->error_message) ? $self->error_message:'';
+        $error_data{error_package} = defined($self->error_package) ? $self->error_package:'';
+        $error_data{error_file} = defined($self->error_file) ? $self->error_file:'';
+        $error_data{error_subroutine} = defined($self->error_subroutine) ? $self->error_subroutine:'';
+        $error_data{error_line} = defined($self->error_line) ? $self->error_line:'';
+        $self->__signal_observers__('error_die', %error_data);
+        die $error if $error;
+    }
 
     $self->is_executed(1);
     $self->result($result);
@@ -122,8 +152,7 @@ sub execute {
     return $result;
 }
 
-sub _execute_body
-{    
+sub _execute_body {
     # default implementation in the base class
     my $self = shift;
     my $class = ref($self) || $self;
@@ -135,15 +164,14 @@ sub _execute_body
 
 
 #
-# Standard external interface for shell dispatchers 
+# Standard external interface for shell dispatchers
 #
 
 # TODO: abstract out all dispatchers for commands into a given API
-sub execute_with_shell_params_and_exit
-{
+sub execute_with_shell_params_and_exit {
     # This automatically parses command-line options and "does the right thing":
     my $class = shift;
-    
+
     if (@_) {
         die
 qq|
@@ -182,8 +210,7 @@ My::Command->execute_with_shell_params_and_exit;
     exit $exit_code;
 }
 
-sub _execute_with_shell_params_and_return_exit_code
-{
+sub _execute_with_shell_params_and_return_exit_code {
     my $class = shift;
     my @argv = @_;
 
@@ -200,7 +227,7 @@ sub _execute_with_shell_params_and_return_exit_code
     } else {
         $rv = $class->_execute_delegate_class_with_params($delegate_class,$params);
     }
-    
+
     my $exit_code = $delegate_class->exit_code_for_return_value($rv);
     return $exit_code;
 }
@@ -217,6 +244,7 @@ sub _execute_delegate_class_with_params {
     $delegate_class->dump_status_messages(1);
     $delegate_class->dump_warning_messages(1);
     $delegate_class->dump_error_messages(1);
+    $delegate_class->dump_usage_messages(1);
     $delegate_class->dump_debug_messages(0);
 
     if ( $delegate_class->is_sub_command_delegator && !defined($params) ) {
@@ -255,10 +283,9 @@ sub _execute_delegate_class_with_params {
 
 #
 # Standard programmatic interface
-# 
+#
 
-sub create 
-{
+sub create {
     my $class = shift;
     my ($rule,%extra) = $class->define_boolexpr(@_);
     my @params_list = $rule->params_list;
@@ -273,8 +300,8 @@ sub create
                 $self->$property_name(0);
             }
         }
-    }    
-    
+    }
+
     return $self;
 }
 
@@ -286,11 +313,11 @@ sub create
 # See above for details of internal implementation.
 
 # By default, there are no bare arguments.
-sub _bare_shell_argument_names { 
+sub _bare_shell_argument_names {
     my $self = shift;
     my $meta = $self->__meta__;
-    my @ordered_names = 
-        map { $_->property_name } 
+    my @ordered_names =
+        map { $_->property_name }
         sort { $a->{shell_args_position} <=> $b->{shell_args_position} }
         grep { $_->{shell_args_position} }
         $self->_shell_args_property_meta();
@@ -302,8 +329,7 @@ sub _bare_shell_argument_names {
 # Also, execute() could return a negative value; this is converted to
 # positive and used as the shell exit code.  NOTE: This means execute()
 # returning 0 and -1 mean the same thing
-sub exit_code_for_return_value 
-{
+sub exit_code_for_return_value {
     my $self = shift;
     my $return_value = shift;
     if (! $return_value) {
@@ -311,13 +337,12 @@ sub exit_code_for_return_value
     } elsif ($return_value < 0) {
         $return_value = 0 - $return_value;
     } else {
-        $return_value = 0 
+        $return_value = 0
     }
     return $return_value;
 }
 
-sub help_brief 
-{
+sub help_brief {
     my $self = shift;
     if (my $doc = $self->__meta__->doc) {
         return $doc;
@@ -339,25 +364,21 @@ sub help_brief
 }
 
 
-sub help_synopsis 
-{
+sub help_synopsis {
     my $self = shift;
     return '';
 }
 
-sub help_detail 
-{
+sub help_detail {
     my $self = shift;
     return "!!! define help_detail() in module " . ref($self) || $self . "!";
 }
 
-sub sub_command_category 
-{
+sub sub_command_category {
     return;
 }
 
-sub sub_command_sort_position 
-{ 
+sub sub_command_sort_position {
     # override to do something besides alpha sorting by name
     return '9999999999 ' . $_[0]->command_name_brief;
 }
@@ -367,16 +388,14 @@ sub sub_command_sort_position
 # Self reflection
 #
 
-sub is_abstract 
-{
+sub is_abstract {
     # Override when writing an subclass which is also abstract.
     my $self = shift;
     my $class_meta = $self->__meta__;
     return $class_meta->is_abstract;
 }
 
-sub is_executable 
-{
+sub is_executable {
     my $self = shift;
     if ($self->can("_execute_body") eq __PACKAGE__->can("_execute_body")) {
         return;
@@ -389,8 +408,7 @@ sub is_executable
     }
 }
 
-sub is_sub_command_delegator
-{
+sub is_sub_command_delegator {
     my $self = shift;
     if (scalar($self->sub_command_dirs)) {
         return 1;
@@ -403,13 +421,12 @@ sub is_sub_command_delegator
 sub _time_now {
     # return the current time in context
     # this may not be the real time in selected cases
-    shift->__context__->now; 
+    shift->__context__->now;
 }
 
-sub color_command_name 
-{
+sub color_command_name {
     my $text = shift;
-    
+
     my $colored_text = [];
 
     my @COLOR_TEMPLATES = ('red', 'bold red', 'magenta', 'bold magenta');
@@ -417,19 +434,17 @@ sub color_command_name
     for(my $i = 0 ; $i < @parts ; $i++ ){
         push @$colored_text, ($i < @COLOR_TEMPLATES) ? Term::ANSIColor::colored($parts[$i], $COLOR_TEMPLATES[$i]) : $parts[$i];
     }
-    
+
     return join(' ', @$colored_text);
 }
 
-sub _base_command_class_and_extension 
-{
+sub _base_command_class_and_extension {
     my $self = shift;
     my $class = ref($self) || $self;
-    return ($class =~ /^(.*)::([^\:]+)$/); 
+    return ($class =~ /^(.*)::([^\:]+)$/);
 }
 
-sub _command_name_for_class_word 
-{
+sub _command_name_for_class_word {
     my $self = shift;
     my $s = shift;
     $s =~ s/_/-/g;
@@ -440,8 +455,7 @@ sub _command_name_for_class_word
     return $s;
 }
 
-sub command_name
-{
+sub command_name {
     my $self = shift;
     my $class = ref($self) || $self;
     my $prepend = '';
@@ -458,8 +472,7 @@ sub command_name
     return $prepend . $n;
 }
 
-sub command_name_brief
-{
+sub command_name_brief {
     my $self = shift;
     my $class = ref($self) || $self;
     my @words = grep { $_ ne 'Command' } split(/::/,$class);
@@ -515,20 +528,19 @@ sub resolve_option_completion_spec {
     return \@completion_spec
 }
 
-sub resolve_class_and_params_for_argv
-{
+sub resolve_class_and_params_for_argv {
     # This is used by execute_with_shell_params_and_exit, but might be used within an application.
     my $self = shift;
     my @argv = @_;
 
     if ($self->is_sub_command_delegator) {
-        if ( $argv[0] and $argv[0] !~ /^\-/ 
+        if ( $argv[0] and $argv[0] !~ /^\-/
                 and my $class_for_sub_command = $self->class_for_sub_command($argv[0]) ) {
             # delegate
             shift @argv;
             return $class_for_sub_command->resolve_class_and_params_for_argv(@argv);
         }
-        
+
         if (@argv) {
             # this has sub-commands, and is also executable
             # fall through to the execution_logic...
@@ -542,7 +554,7 @@ sub resolve_class_and_params_for_argv
             return ($self,undef);
         }
     }
-    
+
     my ($params_hash,@spec) = $self->_shell_args_getopt_specification;
     unless (grep { /^help\W/ } @spec) {
         push @spec, "help!";
@@ -553,12 +565,12 @@ sub resolve_class_and_params_for_argv
     # Not a problem in Perl. :)  (which is probably why it was never fixed)
     local @ARGV;
     @ARGV = @argv;
-   
+
     do {
-        # GetOptions also likes to emit warnings instead of return a list of errors :( 
+        # GetOptions also likes to emit warnings instead of return a list of errors :(
         my @errors;
         local $SIG{__WARN__} = sub { push @errors, @_ };
-        
+
         ## Change the pattern to be '--', '-' followed by a non-digit, or '+'.
         ## This s the effect of treating a negative number as a value of an option.
         ## This means that we won't be allowed to have an option named, say, -1.
@@ -647,7 +659,7 @@ sub help_usage_complete_text {
 
     my $command_name = $self->command_name;
     my $text;
-    
+
     if (not $self->is_executable) {
         # no execute implemented
         if ($self->is_sub_command_delegator) {
@@ -679,29 +691,29 @@ sub help_usage_complete_text {
             "\n%s\n%s\n\n%s%s%s%s%s\n",
             Term::ANSIColor::colored('USAGE', 'underline'),
             Text::Wrap::wrap(
-                ' ', 
-                '    ', 
+                ' ',
+                '    ',
                 Term::ANSIColor::colored($self->command_name, 'bold'),
                 $self->_shell_args_usage_string || '',
             ),
-            ( $synopsis 
+            ( $synopsis
                 ? sprintf("%s\n%s\n", Term::ANSIColor::colored("SYNOPSIS", 'underline'), $synopsis)
                 : ''
             ),
-            ( $required_args 
+            ( $required_args
                 ? sprintf("%s\n%s\n", Term::ANSIColor::colored("REQUIRED ARGUMENTS", 'underline'), $required_args)
                 : ''
             ),
-            ( $optional_args 
+            ( $optional_args
                 ? sprintf("%s\n%s\n", Term::ANSIColor::colored("OPTIONAL ARGUMENTS", 'underline'), $optional_args)
                 : ''
             ),
             sprintf(
-                "%s\n%s\n", 
-                Term::ANSIColor::colored("DESCRIPTION", 'underline'), 
+                "%s\n%s\n",
+                Term::ANSIColor::colored("DESCRIPTION", 'underline'),
                 Text::Wrap::wrap(' ', ' ', $self->help_detail || '')
             ),
-            ( $sub_commands 
+            ( $sub_commands
                 ? sprintf("%s\n%s\n", Term::ANSIColor::colored("SUB-COMMANDS", 'underline'), $sub_commands)
                 : ''
             ),
@@ -730,7 +742,7 @@ sub doc_sections {
 
     push(@sections, UR::Doc::Section->create(
         title => "VERSION",
-        content =>  "This document " # separated to trick the version updater 
+        content =>  "This document " # separated to trick the version updater
             . "describes $command_name "
             . ($version ? "version $version " : "")
             . "($date at $time)",
@@ -781,14 +793,13 @@ sub doc_sections {
     return @sections;
 }
 
-sub help_usage_command_pod
-{
+sub help_usage_command_pod {
     my $self = shift;
 
     my $command_name = $self->command_name;
     my $pod;
 
-    if (0) { # (not $self->is_executable) {
+    if (0) { # (not $self->is_executable)
         # no execute implemented
         if ($self->is_sub_command_delegator) {
             # show the list of sub-commands
@@ -818,15 +829,15 @@ sub help_usage_command_pod
             "\n=pod"
             . "\n\n=head1 NAME"
             .  "\n\n"
-            .   $self->command_name 
-            . ($help_brief ? " - " . $self->help_brief : '') 
+            .   $self->command_name
+            . ($help_brief ? " - " . $self->help_brief : '')
             . "\n\n";
 
         if ($version) {
             $pod .=
                 "\n\n=head1 VERSION"
                 . "\n\n"
-                . "This document " # separated to trick the version updater 
+                . "This document " # separated to trick the version updater
                 . "describes " . $self->command_name . " version " . $version . '.'
                 . "\n\n";
         }
@@ -842,7 +853,7 @@ sub help_usage_command_pod
         else {
             $pod .=
                     (
-                        $synopsis 
+                        $synopsis
                         ? "=head1 SYNOPSIS\n\n" . $synopsis . "\n\n"
                         : ''
                     )
@@ -860,15 +871,14 @@ sub help_usage_command_pod
                 . join('', map { "  $_\n" } split ("\n",$self->help_detail))
                 . "\n";
         }
-        
+
         $pod .= "\n\n=cut\n\n";
 
     }
     return "\n$pod";
 }
 
-sub help_header
-{
+sub help_header {
     my $class = shift;
     return sprintf("%s - %-80s\n",
         $class->command_name
@@ -876,8 +886,7 @@ sub help_header
     )
 }
 
-sub help_options
-{
+sub help_options {
     my $self = shift;
     my %params = @_;
 
@@ -918,7 +927,7 @@ sub help_options
         if ($valid_values) {
             $doc .= "\nvalid values:\n";
             for my $v (@$valid_values) {
-                $doc .= " " . $v . "\n"; 
+                $doc .= " " . $v . "\n";
                 $max_name_length = length($v)+2 if $max_name_length < length($v)+2;
             }
             chomp $doc;
@@ -983,7 +992,7 @@ sub sorted_sub_command_classes {
             ($a->sub_command_sort_position <=> $b->sub_command_sort_position)
             ||
             ($a->sub_command_sort_position cmp $b->sub_command_sort_position)
-        } 
+        }
         @c;
 }
 
@@ -1022,13 +1031,12 @@ sub sub_commands_table {
     return $tb;
 }
 
-sub help_sub_commands
-{
+sub help_sub_commands {
     my $class = shift;
     my %params = @_;
     my $command_name_method = 'command_name_brief';
     #my $command_name_method = ($params{brief} ? 'command_name_brief' : 'command_name');
-    
+
     my @sub_command_classes = $class->sorted_sub_command_classes;
 
     my %categories;
@@ -1043,7 +1051,7 @@ sub help_sub_commands
                 push @categories, $category;
             }
             else {
-                unshift @categories,''; 
+                unshift @categories,'';
             }
             $sub_commands_within_category = $categories{$category} = [];
         }
@@ -1052,7 +1060,7 @@ sub help_sub_commands
 
     no warnings;
     local  $Text::Wrap::columns = 60;
-    
+
     my $full_text = '';
     my @full_data;
     for my $category (@categories) {
@@ -1066,15 +1074,15 @@ sub help_sub_commands
                         $_->_shell_args_usage_string_abbreviated,
                         $rows[0],
                     ],
-                    map { 
-                        [ 
+                    map {
+                        [
                             '',
                             ' ',
                             $rows[$_],
                         ]
                     } (1..$#rows)
                 );
-            } 
+            }
             @$sub_commands_within_this_category;
 
         if ($category) {
@@ -1084,7 +1092,7 @@ sub help_sub_commands
             if ($category =~ /\D/) {
                 # non-numeric categories show their category as a header
                 $category .= ':' if $category =~ /\S/;
-                push @full_data, 
+                push @full_data,
                     [
                         Term::ANSIColor::colored(uc($category), 'blue'),
                         '',
@@ -1118,7 +1126,7 @@ sub help_sub_commands
         }
         $text .= "\n";
     }
-    #$DB::single = 1;        
+    #$DB::single = 1;
     return $text;
 }
 
@@ -1128,13 +1136,12 @@ sub _is_hidden_in_docs { return; }
 # Methods which transform command properties into shell args (getopt)
 #
 
-sub _shell_args_property_meta
-{
+sub _shell_args_property_meta {
     my $self = shift;
     my $class_meta = $self->__meta__;
 
     # Find which property metas match the rules.  We have to do it this way
-    # because just calling 'get_all_property_metas()' will product multiple matches 
+    # because just calling 'get_all_property_metas()' will product multiple matches
     # if a property is overridden in a child class
     my $rule = UR::Object::Property->define_boolexpr(@_);
     my %seen;
@@ -1166,31 +1173,29 @@ sub _shell_args_property_meta
             push @required, $property_meta;
         }
     }
-    
+
     my @result;
-    @result = ( 
+    @result = (
         (sort { $a->property_name cmp $b->property_name } @required),
         (sort { $a->property_name cmp $b->property_name } @optional),
         (sort { $a->{shell_args_position} <=> $b->{shell_args_position} } @positional),
     );
-    
+
     return @result;
 }
 
-sub _shell_arg_name_from_property_meta
-{
+sub _shell_arg_name_from_property_meta {
     my ($self, $property_meta,$singularize) = @_;
     my $property_name = ($singularize ? $property_meta->singular_name : $property_meta->property_name);
     my $param_name = $property_name;
     $param_name =~ s/_/-/g;
-    return $param_name; 
+    return $param_name;
 }
 
-sub _shell_arg_getopt_qualifier_from_property_meta
-{
+sub _shell_arg_getopt_qualifier_from_property_meta {
     my ($self, $property_meta) = @_;
 
-    my $many = ($property_meta->is_many ? '@' : ''); 
+    my $many = ($property_meta->is_many ? '@' : '');
     if (defined($property_meta->data_type) and $property_meta->data_type =~ /Boolean/) {
         return '!' . $many;
     }
@@ -1202,8 +1207,7 @@ sub _shell_arg_getopt_qualifier_from_property_meta
     }
 }
 
-sub _shell_arg_usage_string_from_property_meta 
-{
+sub _shell_arg_usage_string_from_property_meta {
     my ($self, $property_meta) = @_;
     my $string = $self->_shell_arg_name_from_property_meta($property_meta);
     if ($property_meta->{shell_args_position}) {
@@ -1225,7 +1229,7 @@ sub _shell_arg_usage_string_from_property_meta
                 $string .= "=?[,?]";
             }
             else {
-                $string .= '=?'; 
+                $string .= '=?';
             }
             if ($property_meta->is_optional) {
                 $string = "[$string]";
@@ -1235,8 +1239,7 @@ sub _shell_arg_usage_string_from_property_meta
     return $string;
 }
 
-sub _shell_arg_getopt_specification_from_property_meta 
-{
+sub _shell_arg_getopt_specification_from_property_meta {
     my ($self,$property_meta) = @_;
     my $arg_name = $self->_shell_arg_name_from_property_meta($property_meta);
     return (
@@ -1246,8 +1249,7 @@ sub _shell_arg_getopt_specification_from_property_meta
 }
 
 
-sub _shell_arg_getopt_complete_specification_from_property_meta 
-{
+sub _shell_arg_getopt_complete_specification_from_property_meta {
     my ($self,$property_meta) = @_;
     my $arg_name = $self->_shell_arg_name_from_property_meta($property_meta);
     my $completions = $property_meta->valid_values;
@@ -1266,7 +1268,7 @@ sub _shell_arg_getopt_complete_specification_from_property_meta
             'Directory','DirectoryPath','Dir','DirPath',
         );
         if (!defined($type)) {
-            $completions = 'files'; 
+            $completions = 'files';
         }
         else {
             for my $pattern (@complete_as_files) {
@@ -1285,46 +1287,43 @@ sub _shell_arg_getopt_complete_specification_from_property_meta
     }
     return (
         $arg_name .  $self->_shell_arg_getopt_qualifier_from_property_meta($property_meta),
-        $completions, 
+        $completions,
 #        ($property_meta->is_many ? ($arg_name => []) : ())
     );
 }
 
-sub _shell_args_getopt_specification 
-{
+sub _shell_args_getopt_specification {
     my $self = shift;
     my @getopt;
     my @params;
     for my $meta ($self->_shell_args_property_meta) {
         my ($spec, @params_addition) = $self->_shell_arg_getopt_specification_from_property_meta($meta);
         push @getopt,$spec;
-        push @params, @params_addition; 
+        push @params, @params_addition;
     }
     @getopt = sort @getopt;
-    return { @params}, @getopt; 
+    return { @params}, @getopt;
 }
 
-sub _shell_args_getopt_complete_specification
-{
+sub _shell_args_getopt_complete_specification {
     my $self = shift;
     my @getopt;
     for my $meta ($self->_shell_args_property_meta) {
         my ($spec, $completions) = $self->_shell_arg_getopt_complete_specification_from_property_meta($meta);
         push @getopt, $spec, $completions;
     }
-    return @getopt; 
+    return @getopt;
 }
 
-sub _shell_args_usage_string
-{
+sub _shell_args_usage_string {
     my $self = shift;
     if ($self->is_executable) {
         return join(
-            " ", 
-            map { 
-                $self->_shell_arg_usage_string_from_property_meta($_) 
+            " ",
+            map {
+                $self->_shell_arg_usage_string_from_property_meta($_)
             } $self->_shell_args_property_meta()
-            
+
         );
     }
     elsif ($self->is_sub_command_delegator) {
@@ -1337,8 +1336,7 @@ sub _shell_args_usage_string
     return "";
 }
 
-sub _shell_args_usage_string_abbreviated
-{
+sub _shell_args_usage_string_abbreviated {
     my $self = shift;
     if ($self->is_sub_command_delegator) {
         return "...";
@@ -1355,21 +1353,20 @@ sub _shell_args_usage_string_abbreviated
 }
 
 #
-# The following methods build allow a command to determine its 
+# The following methods build allow a command to determine its
 # sub-commands, if there are any.
 #
 
 # This is for cases in which the Foo::Bar command delegates to
 # Foo::Bar::Baz, Foo::Bar::Buz or Foo::Bar::Doh, depending on its paramters.
 
-sub sub_command_dirs
-{
+sub sub_command_dirs {
     my $class = shift;
     my $module = ref($class) || $class;
     $module =~ s/::/\//g;
-    
+
     # multiple dirs is not working quite yet
-    #my @paths = grep { -d $_ } map { "$_/$module"  } @INC; 
+    #my @paths = grep { -d $_ } map { "$_/$module"  } @INC;
     #return @paths;
 
     $module .= '.pm';
@@ -1384,21 +1381,20 @@ sub sub_command_dirs
     return $path;
 }
 
-sub sub_command_classes
-{
+sub sub_command_classes {
     my $class = shift;
     my @paths = $class->sub_command_dirs;
     return unless @paths;
-    @paths = 
-        grep { s/\.pm$// } 
-        map { glob("$_/*") } 
+    @paths =
+        grep { s/\.pm$// }
+        map { glob("$_/*") }
         grep { -d $_ }
-        grep { defined($_) and length($_) } 
+        grep { defined($_) and length($_) }
         @paths;
     return unless @paths;
     my @classes =
         grep {
-            ($_->is_sub_command_delegator or !$_->__meta__->is_abstract) 
+            ($_->is_sub_command_delegator or !$_->__meta__->is_abstract)
         }
         grep { $_ and $_->isa('Command') }
         map { $class->class_for_sub_command($_) }
@@ -1408,16 +1404,14 @@ sub sub_command_classes
     return @classes;
 }
 
-sub sub_command_names
-{
+sub sub_command_names {
     my $class = shift;
     my @sub_command_classes = $class->sub_command_classes;
     my @sub_command_names = map { $_->command_name_brief } @sub_command_classes;
     return @sub_command_names;
 }
 
-sub class_for_sub_command
-{
+sub class_for_sub_command {
     my $self = shift;
     my $class = ref($self) || $self;
     my $sub_command = shift;
@@ -1461,148 +1455,6 @@ sub class_for_sub_command
     }
 }
 
-#
-# Implement error_mesage/warning_message/status_message in a way
-# which handles command-specific callbacks.
-#
-# Build a set of methods for getting/setting/printing error/warning/status messages
-# $class->dump_error_messages(<bool>) Turn on/off printing the messages to STDERR
-#     error and warnings default to on, status messages default to off
-# $class->queue_error_messages(<bool>) Turn on/off queueing of messages
-#     defaults to off
-# $class->error_message("blah"): set an error message
-# $class->error_message() return the last message
-# $class->error_messages()  return all the messages that have been queued up
-# $class->error_messages_arrayref()  return the reference to the underlying
-#     list messages get queued to.  This is the method for truncating the list
-#     or altering already queued messages
-# $class->error_messages_callback(<subref>)  Specify a callback for when error
-#     messages are set.  The callback runs before printing or queueing, so
-#     you can alter @_ and change the message that gets printed or queued
-# And then the same thing for status and warning messages
-
-# The filehandle to print these messages to.  In normal operation this'll just be
-# STDERR, but the test case can change it to capture the messages to somewhere else
-our $stderr = \*STDERR;
-our $stdout = \*STDOUT;
-
-our %msgdata;
-
-sub _get_msgdata {
-    my $self = $_[0];
-    
-    if (ref($self)) {
-        no strict 'refs';
-        my $object_msgdata = $msgdata{$self->id} ||= {};
-        my $class_msgdata = ref($self)->_get_msgdata;
-
-        while (my ($k,$v) = each(%$class_msgdata)) {
-            $object_msgdata->{$k} = $v unless (exists $object_msgdata->{$k});
-        }
-
-        return $object_msgdata;
-    }
-    else {
-        no strict 'refs';
-        return ${ $self . "::msgdata" } ||= {};
-    }
-}
-
-for my $type (qw/error warning status debug usage/) {
-
-    for my $method_base (qw/_messages_callback queue_ dump_ _package _file _line _subroutine/) {
-        my $method = (substr($method_base,0,1) eq "_"
-            ? $type . $method_base
-            : $method_base . $type . "_messages"
-        );
-        my $method_subref = sub {
-            my $self = shift;
-            my $msgdata = $self->_get_msgdata;
-            $msgdata->{$method} = pop if @_;
-            return $msgdata->{$method};
-        };
-        no strict;
-        no warnings;
-        *$method = $method_subref;
-    }
-
-    my $logger_subname = $type . "_message";
-    my $logger_subref = sub {
-        my $self = shift;
-
-        my $msgdata = $self->_get_msgdata();
-
-        if (@_) {
-            my $msg = shift;
-            chomp $msg if defined $msg;
-
-            unless (defined ($msgdata->{'dump_' . $type . '_messages'})) {
-                $msgdata->{'dump_' . $type . '_messages'} = $type eq "status" ? (exists $ENV{'UR_COMMAND_DUMP_STATUS_MESSAGES'} && $ENV{'UR_COMMAND_DUMP_STATUS_MESSAGES'} ? 1 : 0) : 1;
-            }
-
-            if (my $code = $msgdata->{ $type . "_messages_callback"}) {
-                $code->($self,$msg);
-            }
-            if (my $fh = $msgdata->{ "dump_" . $type . "_messages" }) {
-                if ( $type eq 'usage' ) {
-                    (ref($fh) ? $fh : $stdout)->print((($type eq "status" or $type eq 'usage') ? () : (uc($type), ": ")), (defined($msg) ? $msg : ""), "\n");
-                }
-                else {
-                    (ref($fh) ? $fh : $stderr)->print((($type eq "status" or $type eq 'usage') ? () : (uc($type), ": ")), (defined($msg) ? $msg : ""), "\n");
-                }
-            }
-            if ($msgdata->{ "queue_" . $type . "_messages"}) {
-                my $a = $msgdata->{ $type . "_messages_arrayref" } ||= [];
-                push @$a, $msg;
-            }
-            $msgdata->{ $type . "_message" } = $msg;
-
-            my ($package, $file, $line, $subroutine) = caller;
-            $msgdata->{ $type . "_package" } = $package;
-            $msgdata->{ $type . "_file" } = $file;
-            $msgdata->{ $type . "_line" } = $line;
-            $msgdata->{ $type . "_subroutine" } = $subroutine;
-        }
-
-        if (wantarray) {
-            return (
-                $msgdata->{ $type . "_message" },
-                $msgdata->{ $type . "_package" },
-                $msgdata->{ $type . "_file" },
-                $msgdata->{ $type . "_line" },
-                $msgdata->{ $type . "_subroutine"},
-            );
-        }
-        return $msgdata->{ $type . "_message" };
-    };
-
-
-    my $arrayref_subname = $type . "_messages_arrayref";
-    my $arrayref_subref = sub {
-        my $self = shift;
-        my $msgdata = $self->_get_msgdata;
-        return $msgdata->{$type . "_messages_arrayref"};
-    };
-
-
-    my $array_subname = $type . "_messages";
-    my $array_subref = sub {
-        my $self = shift;
-
-        my $msgdata = $self->_get_msgdata;
-        return ref($msgdata->{$type . "_messages_arrayref"}) ?
-               @{ $msgdata->{$type . "_messages_arrayref"} } :
-               ();
-    };
-
-    no strict;
-    no warnings;
-
-    *$logger_subname    = $logger_subref;
-    *$arrayref_subname  = $arrayref_subref;
-    *$array_subname     = $array_subref;
-}
-
 # Run the given command-line with stdout and stderr redirected to /dev/null
 sub system_inhibit_std_out_err {
     my($self,$cmdline) = @_;
@@ -1677,7 +1529,7 @@ Command - base class for modules implementing the command pattern
   }
 
   # Another part of the code
- 
+
   my $cmd = TopLevelNamespace::SomeObj::Command->create(some_obj_id => $some_obj->id);
   $cmd->execute();
 
@@ -1701,7 +1553,7 @@ and a script called tln_cmd that looks like:
   TopLevelNamespace::Command->execute_with_shell_params_and_exit();
 
 gives you an instant command-line tool as an interface to the hierarchy of
-command modules at TopLevelNamespace::Command.  
+command modules at TopLevelNamespace::Command.
 
 For example:
 
@@ -1718,17 +1570,14 @@ The infrastructure takes care of turning the command line parameters into
 parameters for create().  Params designated as is_optional are, of course,
 optional and non-optional parameters that are missing will generate an error.
 
---help is an implicit param applicable to all Command modules.  It generates 
+--help is an implicit param applicable to all Command modules.  It generates
 some hopefully useful text based on the documentation in the class definition
 (the 'doc' attributes you can attach to a class and properties), and the
 strings returned by help_detail(), help_brief() and help_synopsis().
 
 =head1 TODO
 
-This documentation needs to be fleshed out more.  There's a lot of special 
+This documentation needs to be fleshed out more.  There's a lot of special
 things you can do with Command modules that isn't mentioned here yet.
 
 =cut
-
-
-
