@@ -26,7 +26,7 @@ Or write the singleton to represent the source directly:
 =cut
 
 require UR;
-our $VERSION = "0.35"; # UR $VERSION;
+our $VERSION = "0.36"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => 'UR::DataSource::SQLite',
@@ -761,6 +761,36 @@ sub _get_foreign_key_setting {
     return $foreign_key_setting{$id};
 }
 
+sub resolve_order_by_clause {
+    my($self,$order_by_columns,$order_by_column_data) = @_;
+
+    my @cols = @$order_by_columns;
+    foreach my $col ( @cols) {
+        my $is_descending;
+        if ($col =~ m/^(-|\+)(.*)$/) {
+            $col = $2;
+            if ($1 eq '-') {
+                $is_descending = 1;
+            }
+        }
+
+        my $property_meta = $order_by_column_data->{$col} ? $order_by_column_data->{$col}->[1] : undef;
+        my $is_optional; $is_optional = $property_meta->is_optional if $property_meta;
+
+        if ($is_optional) {
+            if ($is_descending) {
+                $col = "CASE WHEN $col ISNULL THEN 0 ELSE 1 END, $col DESC";
+            } else {
+                $col = "CASE WHEN $col ISNULL THEN 1 ELSE 0 END, $col";
+            }
+        } elsif ($is_descending) {
+            $col = $col . ' DESC';
+        }
+    }
+    return  'order by ' . join(', ',@cols);
+}
+
+
 sub _dump_db_to_file_internal {
     my $self = shift;
 
