@@ -3,7 +3,7 @@ package UR::DeletedRef;
 use strict;
 use warnings;
 require UR;
-our $VERSION = "0.35"; # UR $VERSION;
+our $VERSION = "0.36"; # UR $VERSION;
 
 our $all_objects_deleted = {};
 
@@ -48,7 +48,7 @@ sub AUTOLOAD {
     our $AUTOLOAD;
     my $method = $AUTOLOAD;
     $method =~ s/^.*:://g;
-    Carp::confess("Attempt to use a reference to an object which has been deleted.  A call was made to method '$method'\nRessurrect it first.\n" . Dumper($_[0]));
+    Carp::croak("Attempt to use a reference to an object which has been deleted.  A call was made to method '$method'\nRessurrect it first.\n" . Dumper($_[0]));
 }
 
 sub DESTROY {
